'use strict';

//----------------------------------------------------------------------
//
// This source file is part of the Folktale project.
//
// Licensed under MIT. See LICENCE for full licence information.
// See CONTRIBUTORS for the list of contributors to the project.
//
//----------------------------------------------------------------------

var _require = require('../helpers/fantasy-land'),
    flAlt = _require.alt;

var curry = require('../core/lambda/curry');
var warn = require('../helpers/warn-deprecated-method')('alt');
var unsupported = require('../helpers/unsupported-method')('alt');

var isNew = function isNew(a) {
  return typeof a[flAlt] === 'function';
};
var isOld = function isOld(a) {
  return typeof a.alt === 'function';
};

/*~
 * stability: stable
 * authors:
 *   - "@joshmili"
 *
 * type: |
 *   forall F, a:
 *     (F a, F a) => F a
 *   where F is Alt
 */
var alt = function alt(aAlt, bAlt) {
  return isNew(bAlt) ? bAlt[flAlt](aAlt) : isOld(aAlt) ? warn(aAlt.ap(bAlt)) : /*otherwise*/unsupported(aAlt);
};

/*~
 * stability: stable
 * authors:
 *   - "@joshmili"
 *
 * type: |
 *   forall F, a:
 *     (F a) => (F a) => F a
 *   where F is Alt
 */
alt.curried = curry(2, alt); // eslint-disable-line no-magic-numbers


/*~
 * stability: experimental
 * authors:
 *   - "@joshmili"
 *
 * type: |
 *   forall F, a:
 *     (F a).(F a) => F a
 *   where F is Alt
 */
alt.infix = function (aAlt) {
  return alt(this, aAlt);
};

module.exports = alt;